@@ -121,14 +121,31 @@ class Response:
             return read_size
         return self.socket.recv_into(buf, size)
 
+    @staticmethod
+    def _find(buf, needle, start, end):
+        if hasattr(buf, "find"):
+            return buf.find(needle, start, end)
+        result = -1
+        i = start
+        while i < end:
+            j = 0
+            while j < len(needle) and i + j < end and buf[i + j] == needle[j]:
+                j += 1
+            if j == len(needle):
+                result = i
+                break
+            i += 1
+
+        return result
+
     def _readto(self, first, second=b""):
         buf = self._receive_buffer
         end = self._received_length
         while True:
-            firsti = buf.find(first, 0, end)
+            firsti = self._find(buf, first, 0, end)
             secondi = -1
             if second:
-                secondi = buf.find(second, 0, end)
+                secondi = self._find(buf, second, 0, end)
 
             i = -1
             needle_len = 0
@@ -318,7 +335,12 @@ class Response:
         if not self._raw:
             self._raw = _RawResponse(self)
 
-        obj = json.load(self._raw)
+        try:
+            obj = json.load(self._raw)
+        except OSError:
+            # <5.3.1 doesn't piecemeal load json from any object with readinto so load the whole
+            # string.
+            obj = json.loads(self._raw.read())
         if not self._cached:
             self._cached = obj
         self.close()
@@ -355,26 +377,28 @@ class Session:
         self._last_response = None
 
     def _free_socket(self, socket):
-
         if socket not in self._open_sockets.values():
             raise RuntimeError("Socket not from session")
         self._socket_free[socket] = True
 
+    def _close_socket(self, sock):
+        sock.close()
+        del self._socket_free[sock]
+        key = None
+        for k in self._open_sockets:
+            if self._open_sockets[k] == sock:
+                key = k
+                break
+        if key:
+            del self._open_sockets[key]
+
     def _free_sockets(self):
         free_sockets = []
         for sock in self._socket_free:
             if self._socket_free[sock]:
-                sock.close()
                 free_sockets.append(sock)
         for sock in free_sockets:
-            del self._socket_free[sock]
-            key = None
-            for k in self._open_sockets:
-                if self._open_sockets[k] == sock:
-                    key = k
-                    break
-            if key:
-                del self._open_sockets[key]
+            self._close_socket(sock)
 
     def _get_socket(self, host, port, proto, *, timeout=1):
         key = (host, port, proto)
@@ -418,6 +442,61 @@ class Session:
         self._socket_free[sock] = False
         return sock
 
+    @staticmethod
+    def _send(socket, data):
+        total_sent = 0
+        while total_sent < len(data):
+            sent = socket.send(data[total_sent:])
+            if sent is None:
+                sent = len(data)
+            if sent == 0:
+                raise RuntimeError("Connection closed")
+            total_sent += sent
+
+    def _send_request(self, socket, host, method, path, headers, data, json):
+        # pylint: disable=too-many-arguments
+        self._send(socket, bytes(method, "utf-8"))
+        self._send(socket, b" /")
+        self._send(socket, bytes(path, "utf-8"))
+        self._send(socket, b" HTTP/1.1\r\n")
+        if "Host" not in headers:
+            self._send(socket, b"Host: ")
+            self._send(socket, bytes(host, "utf-8"))
+            self._send(socket, b"\r\n")
+        if "User-Agent" not in headers:
+            self._send(socket, b"User-Agent: Adafruit CircuitPython\r\n")
+        # Iterate over keys to avoid tuple alloc
+        for k in headers:
+            self._send(socket, k.encode())
+            self._send(socket, b": ")
+            self._send(socket, headers[k].encode())
+            self._send(socket, b"\r\n")
+        if json is not None:
+            assert data is None
+            # pylint: disable=import-outside-toplevel
+            try:
+                import json as json_module
+            except ImportError:
+                import ujson as json_module
+            data = json_module.dumps(json)
+            self._send(socket, b"Content-Type: application/json\r\n")
+        if data:
+            if isinstance(data, dict):
+                self._send(
+                    socket, b"Content-Type: application/x-www-form-urlencoded\r\n"
+                )
+                _post_data = ""
+                for k in data:
+                    _post_data = "{}&{}={}".format(_post_data, k, data[k])
+                data = _post_data[1:]
+            self._send(socket, b"Content-Length: %d\r\n" % len(data))
+        self._send(socket, b"\r\n")
+        if data:
+            if isinstance(data, bytearray):
+                self._send(socket, bytes(data))
+            else:
+                self._send(socket, bytes(data, "utf-8"))
+
     # pylint: disable=too-many-branches, too-many-statements, unused-argument, too-many-arguments, too-many-locals
     def request(
         self, method, url, data=None, json=None, headers=None, stream=False, timeout=60
@@ -454,42 +533,11 @@ class Session:
             self._last_response = None
 
         socket = self._get_socket(host, port, proto, timeout=timeout)
-        socket.send(
-            b"%s /%s HTTP/1.1\r\n" % (bytes(method, "utf-8"), bytes(path, "utf-8"))
-        )
-        if "Host" not in headers:
-            socket.send(b"Host: %s\r\n" % bytes(host, "utf-8"))
-        if "User-Agent" not in headers:
-            socket.send(b"User-Agent: Adafruit CircuitPython\r\n")
-        # Iterate over keys to avoid tuple alloc
-        for k in headers:
-            socket.send(k.encode())
-            socket.send(b": ")
-            socket.send(headers[k].encode())
-            socket.send(b"\r\n")
-        if json is not None:
-            assert data is None
-            # pylint: disable=import-outside-toplevel
-            try:
-                import json as json_module
-            except ImportError:
-                import ujson as json_module
-            data = json_module.dumps(json)
-            socket.send(b"Content-Type: application/json\r\n")
-        if data:
-            if isinstance(data, dict):
-                socket.send(b"Content-Type: application/x-www-form-urlencoded\r\n")
-                _post_data = ""
-                for k in data:
-                    _post_data = "{}&{}={}".format(_post_data, k, data[k])
-                data = _post_data[1:]
-            socket.send(b"Content-Length: %d\r\n" % len(data))
-        socket.send(b"\r\n")
-        if data:
-            if isinstance(data, bytearray):
-                socket.send(bytes(data))
-            else:
-                socket.send(bytes(data, "utf-8"))
+        try:
+            self._send_request(socket, host, method, path, headers, data, json)
+        except:
+            self._close_socket(socket)
+            raise
 
         resp = Response(socket, self)  # our response
         if "location" in resp.headers and 300 <= resp.status_code <= 399:
@@ -535,6 +583,7 @@ class _FakeSSLSocket:
         self.settimeout = socket.settimeout
         self.send = socket.send
         self.recv = socket.recv
+        self.close = socket.close
 
     def connect(self, address):
         """connect wrapper to add non-standard mode parameter"""
